@@ -513,7 +513,6 @@ color_settings_profile_add_cb (GtkButton *button, ColorSettings *settings)
     window_title = g_strdup_printf ("Add Color Profile to %s",
                                     color_device_get_kind (settings->current_device) ? color_device_get_kind (settings->current_device) : "Device");
     gtk_window_set_title (GTK_WINDOW (settings->dialog_assign), window_title);
-    xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (settings->dialog_assign), color_device_get_title (settings->current_device));
     g_free (window_title);
 
     response = gtk_dialog_run (GTK_DIALOG (settings->dialog_assign));
@@ -1403,7 +1402,7 @@ main (gint argc, gchar **argv)
     /* check if we should print version information */
     if (G_UNLIKELY (opt_version)) {
         g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
-        g_print ("%s\n", "Copyright (c) 2008-2019");
+        g_print ("%s\n", "Copyright (c) 2008-2022");
         g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print ("\n");
@@ -1425,7 +1424,6 @@ main (gint argc, gchar **argv)
         if (G_UNLIKELY (opt_socket_id == 0)) {
             /* Get the dialog widget */
             settings->dialog = gtk_builder_get_object (builder, "dialog");
-            gtk_window_set_type_hint (GTK_WINDOW (settings->dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
 
             g_signal_connect (settings->dialog, "response",
                               G_CALLBACK (color_settings_dialog_response), settings);
@@ -1439,7 +1437,8 @@ main (gint argc, gchar **argv)
         else {
             /* Create plug widget */
             plug = gtk_plug_new (opt_socket_id);
-            g_signal_connect (plug, "delete-event", G_CALLBACK (color_settings_dialog_destroy), settings);
+            g_signal_connect_swapped (plug, "delete-event",
+                                      G_CALLBACK (color_settings_dialog_destroy), settings);
             gtk_widget_show (plug);
 
             /* Stop startup notification */
